import sys
import requests
import datetime, time
import json

def main(argv):
    apiKey = argv[0]
    apiSecret = argv[1]
    (accessToken, expiresIn) = getAccessToken(apiKey, apiSecret)
    print 'Access token: ' + accessToken
    print 'Expires in: ' + str(expiresIn)
    
    expirationTime = getExpirationTime(expiresIn)
    
    r'Need to obtain a new access token before the current one expires.'
    if isTokenExpiring(expirationTime):
        r'Because there is no refresh_token for this grant type, need to get a new token in the same way'
        (accessToken, expiresIn) = getAccessToken(apiKey, apiSecret)
    
def getAccessToken(apiKey, apiSecret):
    stsUrl = 'https://login.myacxiom.com/api/v1/auth/oauth2/token'
    payload= {'client_id' : apiKey,'client_secret':apiSecret, 'grant_type':'client_credentials'}
    
    response = requests.post(stsUrl, data=payload)
    if response.status_code == 200:
        data = json.loads(response.text)
        accessToken = data['access_token']
        expiresIn = data['expires_in'] #in seconds
        return (accessToken, expiresIn)
    else:
        print 'Failed to obtain access token, status code ' + str(response.status_code)
        return ('', '0')

def getExpirationTime(expiresIn):
    r'We recommend not waiting until token expiration before getting a new one due to network transfer time involved.'
    bufferTime = 60
    currentTime = datetime.datetime.now()
    unixTimestamp = time.mktime(currentTime.timetuple())
    expiresAt = unixTimestamp + int(expiresIn) - bufferTime
    print 'Token is expiring at: ' + str(expiresAt)
    
    return expiresAt
            
def isTokenExpiring(expiresAt):
    currentTime = datetime.datetime.now()
    unixTimestamp = time.mktime(currentTime.timetuple())
    print 'Current timestamp: ' + str(unixTimestamp)
    
    if unixTimestamp >= expiresAt:
        return True
    
    return False
    
if __name__ == '__main__':
    if len(sys.argv) != 3:
        print "Usage: python ClientCredentials.py apiKey, apiSecret"
        sys.exit(1)
    main(sys.argv[1:])
    
    