import sys
import requests
import datetime, time
import json

def main(argv):
    apiKey = argv[0]
    apiSecret = argv[1]
    tenantSystemName = argv[2]
    userName = argv[3]
    userPassword = argv[4]
    
    (accessToken, expiresIn, refreshToken) = getAccessToken(apiKey, apiSecret, tenantSystemName+"\\"+userName, userPassword)
    print 'Access token: ' + accessToken
    print 'Expires in: ' + str(expiresIn)
    print 'Refresh token: ' + refreshToken
    
    expirationTime = getExpirationTime(expiresIn)
    
    r'Need to obtain a new access token before the current one expires.'
    if isTokenExpiring(expirationTime):
        (accessToken, expiresIn, newRefreshToken) = renewToken(apiKey, apiSecret, refreshToken)
        print 'Access token: ' + accessToken
        print 'Expires in: ' + str(expiresIn)
        print 'Refresh token: ' + newRefreshToken
    
def getAccessToken(apiKey, apiSecret, user, password):
    stsUrl = 'https://login.myacxiom.com/api/v1/auth/oauth2/token'
    payload= {'client_id' : apiKey,'client_secret':apiSecret, 'username':user, 'password':password, 'grant_type':'password'}
    
    response = requests.post(stsUrl, data=payload)
    if response.status_code == 200:
        data = json.loads(response.text)
        accessToken = data['access_token']
        expiresIn = data['expires_in'] #in seconds
        refreshToken = data['refresh_token']
        return (accessToken, expiresIn, refreshToken)
    else:
        print 'Failed to obtain access token, status code ' + str(response.status_code)
        return ('', '0', '')

def renewToken(apiKey, apiSecret, refreshToken):
    stsUrl = 'https://login.myacxiom.com/api/v1/auth/oauth2/token'
    payload= {'client_id' : apiKey,'client_secret':apiSecret, 'grant_type':'refresh_token', 'refresh_token':refreshToken}
    
    response = requests.post(stsUrl, data=payload)
    if response.status_code == 200:
        data = json.loads(response.text)
        accessToken = data['access_token']
        expiresIn = data['expires_in'] #in seconds
        refreshToken = data['refresh_token']
        return (accessToken, expiresIn, refreshToken)
    else:
        print 'Failed to refresh access token, status code ' + str(response.status_code)
        return ('', '0', '')
    
def getExpirationTime(expiresIn):
    r'We recommend not waiting until token expiration before getting a new one due to network transfer time involved.'
    bufferTime = 60
    currentTime = datetime.datetime.now()
    unixTimestamp = time.mktime(currentTime.timetuple())
    expiresAt = unixTimestamp + expiresIn - bufferTime
    print 'Token is expiring at: ' + str(expiresAt)
    
    return expiresAt
            
def isTokenExpiring(expiresAt):
    currentTime = datetime.datetime.now()
    unixTimestamp = time.mktime(currentTime.timetuple())
    print 'Current timestamp: ' + str(unixTimestamp)
    
    if unixTimestamp >= expiresAt:
        return True
    
    return False
    
if __name__ == '__main__':
    if len(sys.argv) != 6:
        print "Usage: python Password.py apiKey, apiSecret, tenantSystemName, userName, password"
        sys.exit(1)
    main(sys.argv[1:])
